#ifndef  __MAIN_DEFINED
#define __MAIN_DEFINED

#include <cdefBF533.h>
#include <ccblkfn.h>
#include <sys\exception.h>




#define DSPID                  0xFFE05000  // DSP Processor ID Register for MP implementations
#define pDSPID ((volatile unsigned long *)DSPID)

#define TC_PER 			0xFFC00B0C
#define pTC_PER 		(volatile unsigned short *)TC_PER


/********************************************************************************/
/***** Prototypes															*****/
/********************************************************************************/
void InitSDRAM(void);
void Interrupt_Init(void);
void Exception_Init(void);
void InitCOMET(void);
void InitHardware(void);
void Sport0_Init(void);
void Sport0_Enable(void);
void Sport0_Disable(void);
void UART_Init(void);
void Process_Data(void);





/********************************************************************************/
/***** Global variables														*****/
/***** All global variables are declared in main.c							*****/
/********************************************************************************/
extern volatile short sSport0RxBuffer[], sSport0TxBuffer[];

struct sDMA_descriptor {struct sDMA_descriptor* next_descriptor;
					    volatile short *start_address;
};
typedef struct sDMA_descriptor tDMA_descriptor;


extern tDMA_descriptor DMA_TX_first; // declaration
extern tDMA_descriptor DMA_RX_first; // declaration


extern volatile bool new_block_received;					// indicates that a full block has been received
extern volatile bool block_transmitted;					// indicates that a processed block has been transmitted

extern volatile int Number_of_received_blocks;			// counter of received blocks
extern volatile int Number_of_transmitted_blocks;		// counter of transmitted blocks

extern short Global_Status;
extern short Per_Channel_Status[];

extern short RIN_scaling_factor[];
extern short SIN_scaling_factor[];

/********************************************************************************/
/***** Symbolic constants													*****/
/********************************************************************************/

// Compilation & debugging options
//#define STANDALONE			// for version to be burned into flash (i.e. no STDIO)
#define DUMP_VARIABLES			// dumps message with addresses of useful variables over the UART terminal
#define DEBUG_ALGORITHM			// uses flags for execution time signalling etc.
//#define DEBUG_COMET			// checks every write to the COMET registers
//#define SKIP_AUTOBAUD_DETECTION		// skips autobaud detection and sets divisor for 115kbaud
#define	PF_AS_CHIP_SELECT		// for prototye: OE\ must be set and cleared "manually"


// system constants  
//#define CLKIN	  (33.33333e6)	// clockin frequency in Hz
#define CLKIN	  (27e6)	// clockin frequency in Hz
#define CORECLK  (600e6)		// core clock frequency in Hz
#define SYSCLK	  (120e6)		// system clock frequency in Hz
#define SPORT0CLK	  (2.048e6)		// SPORT0 (backplane) clock frequency in Hz


#define Number_of_Samples   40		// 40 samples/channel

#define Number_of_Channels  32			// 32 channels
#define Word_Size  			2			// data transfer size in Bytes

#define default_RIN_input_scaling	0	// startup scaling of RIN (number of bits to be shifted up)
#define default_SIN_input_scaling	2	// startup scaling of SIN (number of bits to be shifted up)




// change this to set companding mode
#define SOFTWARE_COMPANDING			// comment if SPORT hardware companding is desired
									// ATTENTION: HARDWARE companding does NOT preserve bit-accuracy

// The following settings select HARDWARE companding ONLY
// Software companding is selected at RUN-time through a control variable (see above)
#ifndef SOFTWARE_COMPANDING
	#define COMPANDING_LAW 	0xC			// A_LAW setting
	//#define COMPANDING_LAW 	0x8			// MU_LAW setting
	//#define COMPANDING_LAW 	0x0			// No companding setting
#endif


// Operating states of channels
// ENUMs
enum { PASS_THROUGH, ENABLED };		// global setting:
									//  echo canceller off for all channels 
									//  echo canceller enabled according to per-channel settings

enum { OFF, ON };					// per-channel setting:
									//  echo canceller off (passing input to output)
									//  echo canceller enabled 

#ifdef SOFTWARE_COMPANDING
	enum { SOFTWARE_MU_LAW, SOFTWARE_A_LAW };	// global setting:
											//  process data according to mu-law
											//  process data according to a-law
	extern short Software_Companding;
#endif


// Background Telemetry Channel (not implemented yet)
//#define BTC_monitor						// include Background Telemetry Channel


// Framing Format  (choose one)
#define T1_map			// defines settings for T1 (mapping T1 onto a 2.048MHz backplane)
//#define E1				// defines settings for E1 @ 2.048MHz (not implemented yet)
//#define T1				// defines settings for T1 @ 1.544MHz (not implemented yet)


// Bit Clock and Bit Frame settings (choose one of each)
//#define CLOCK_CONF1				// Both Comets have clocks as inputs, clock is from SPORT
#define CLOCK_CONF2				// Both Comets have clocks as inputs, clock is from oscillator

#define FRAME_CONF1				// Both Comets have frame sync as inputs, SPORT generates FS
//#define FRAME_CONF2				// Comet1 generates frame sync, comet2 and SPORT have FS as inputs (not implemented)


#define COMET_DATA_EDGE_T		(0)			// 1 ==Data sampled on rising edge		(transmit backplane)
#define COMET_FRAME_EDGE_T	(1)			// 1 ==Frame sampled on rising edge		(transmit backplane)
#define COMET_DATA_EDGE_R		(1)			// 1 ==Data sampled on rising edge		(receive backplane)
#define COMET_FRAME_EDGE_R	(1)			// 1 ==Frame sampled on rising edge		(receive backplane)

#define SPORT_EDGE_R		(0)			// 0 ==Frame driven on rising data sampled on falling edge		(receive section)
#define SPORT_EDGE_T		(0)			// 0 ==Frame and data driven on rising edge		(transmit section)



// General Purpose Flags
#define COMET_RESET							0	// Comets Hardware reset pin assignment (output)
#define EXECUTION_TIME_ALL_CHANNELS			1	// Flag indicating total execution time for the algorithm (all channels)
#define COMET_INTERRUPT_FLAG				2	// Input Flag indicating that at least one of the COMETs has requested an interrupt.
#define CONVERGENT_CHANNEL					3	// Flag indicating that currently processed channel is converged
#define CHIP_SELECT_PF						4	// Flag used to disable Buffers on Adress and Data buses during SDRAM accesses


// COMET DEVICES base addresses  (See EZ-kit and Hardware reference manuals for memory mapping of ASYNC banks)
#define DEVICE1				0x20300000	// Base Address for Device 1 (RIN, SOUT)	(Bank 3 in external async memory)
#define DEVICE2				0x20100000	// Base Address for Device 2 (SIN, ROUT)	(Bank 1 in external async memory)


#endif

